<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xs="http://www.w3.org/2001/XMLSchema"
		xmlns:p5="http://www.tei-c.org/ns/1.0"
		xmlns:xh="http://www.w3.org/1999/xhtml"
		xmlns="http://www.w3.org/1999/xhtml"
		version="1.0">

  <xsl:import href="p5_to_html.xsl"/>
  <!-- import href="../../../uyghur.ittc.ku.edu/lib/p5_to_html.xsl" -->

  <xsl:output encoding="us-ascii"/>

  <!--* lingua = anglica, theodisca, ambae *-->
  <!--
  <xsl:param name="lingua" select=" 'theodisca' "/>
  <xsl:param name="lingua" select=" 'anglica' "/>
  -->
  <xsl:param name="lingua" select=" 'ambae' "/>

  <xsl:param name="num-sec" select=" 'yes' "/>


  <xsl:template name="inject-css-link">
    <xsl:element name="link" namespace="{$xhtmlns}">
      <xsl:attribute name="rel">stylesheet</xsl:attribute>
      <xsl:attribute name="href">local.css</xsl:attribute>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:div[@type='abstract']">
    <xsl:apply-imports/>
    <xsl:element name="hr" namespace="{$xhtmlns}"/>
  </xsl:template>
  
  <xsl:template match="p5:front[not(p5:divGen[@type='toc'])]">
    <xsl:apply-imports/>
    <xsl:call-template name="generate-toc"/>
    <xsl:element name="hr" namespace="{$xhtmlns}"/>
  </xsl:template>

  <xsl:template match="p5:list[@type='simple']">
    <xsl:call-template name="bulleted-list"/>
  </xsl:template>

  <xsl:template match="p5:item/p5:p">
    <xsl:element name="p" namespace="{$xhtmlns}">
      <xsl:attribute name="class">p</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="p5:figure">
    <xsl:element name="div" namespace="{$xhtmlns}">
      <xsl:attribute name="class">figure</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:figure/p5:graphic">
    <xsl:element name="img" namespace="{$xhtmlns}">
      <xsl:attribute name="src"><xsl:value-of select="@url"/></xsl:attribute>
      <xsl:choose>
	<xsl:when test="@rend and
			normalize-space(translate(@rend,'0123456789.%',''))
			= ''">
	  <xsl:attribute name="style">
	    <xsl:value-of select="concat('width: ', @rend, ';
	      text-align: center;')"/>
	  </xsl:attribute>
	</xsl:when>
	<xsl:when test="ancestor::p5:p[@rend = 'slide']">
	  <xsl:attribute name="style">
	    <xsl:text>width: 20%; float: right; padding-right: 1em;</xsl:text>
	  </xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:p[@rend='slide']/p5:figure/p5:p"/>

  
  <xsl:template match="p5:figure/p5:head">
    <xsl:element name="p" namespace="{$xhtmlns}">
      <xsl:attribute name="class">caption</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
    
  <xsl:template match="p5:p/p5:label">
    <xsl:element name="span" namespace="{$xhtmlns}">
      <xsl:attribute name="class">label</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <!--* translation pairs *-->    
  <xsl:template match="p5:div[@type = 'pair']|p5:div[not(@type) and not(p5:head)]">
    <xsl:element name="div" namespace="{$xhtmlns}">
      <xsl:attribute name="class">pair</xsl:attribute>
      <xsl:choose>
	  <xsl:when test="$lingua = 'ambae'">
	    <xsl:apply-templates/>
	  </xsl:when>
	  <xsl:when test="$lingua = 'anglica' ">
	    <xsl:apply-templates mode="en-only"/>
	  </xsl:when>
	  <xsl:when test="$lingua = 'theodisca' ">
	    <xsl:apply-templates mode="de-only"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:apply-templates/>
	  </xsl:otherwise>
      </xsl:choose>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:div[@type = 'pair' or 
		       (not(@type) and
		       not(p5:head))]
		       /p5:p[@xml:lang='en' or (following-sibling::*
		       and not(@xml:lang)) ]"
		mode="de-only"
		/>
  <xsl:template match="p5:p[normalize-space(.) =  'Ø' ]"
		priority="10"
		mode="de-only"
		/>
  <xsl:template match="p5:div[@type = 'pair' or 
		       (not(@type) and
		       not(p5:head))]/p5:p[
		       @xml:lang='de' or (preceding-sibling::*
		       and not(@xml:lang))]"
		mode="en-only"
		/>
  <xsl:template match="p5:p[normalize-space(.) =  'Ø' ]"
		priority="10"
		mode="en-only"
		/>
  
  <xsl:template match="p5:div[@type = 'pair' or 
		       (not(@type) and not(p5:head))]/p5:p">
    <xsl:element name="div" namespace="{$xhtmlns}">
      <xsl:attribute name="class">
	<xsl:text>para </xsl:text>
	<xsl:choose>
	  <xsl:when test="following-sibling::*">
	    <xsl:text>en</xsl:text>
	  </xsl:when>
	  <xsl:when test="preceding-sibling::*">
	    <xsl:text>de</xsl:text>
	  </xsl:when>
	</xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:head/p5:seg[@xml:lang='en'][following-sibling::p5:seg[@xml:lang='de']]">
    <xsl:element name="span" namespace="{$xhtmlns}">
      <xsl:attribute name="class">en</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
    <xsl:text>&#xA0;&#183;&#xA0;</xsl:text>
  </xsl:template>

  <!--* ad hoc rule to suppress bogus entries in toc for translation
      pair divs *-->  
    
    <xsl:template match="p5:div[@type='pair' or not(p5:head)]" mode="toc"/>

    <!--* slide signals *-->
    <xsl:template match="p5:p[@rend='slide']">
    <xsl:element name="div" namespace="{$xhtmlns}">
      <xsl:attribute name="class">slide-signal</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

    <!--* superscripts *-->
    <xsl:template match="p5:hi[@rend='sup']">
    <xsl:element name="sup" namespace="{$xhtmlns}">
      <xsl:apply-templates/>
    </xsl:element>
    </xsl:template>

    <!--* cuts and rewordings *-->  
    <xsl:template match="p5:del" priority="10"
		  mode="de-only"
		  />
  <xsl:template match="p5:del">
    <xsl:element name="del" namespace="{$xhtmlns}">
      <xsl:attribute name="class">del</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  <xsl:template match="p5:add">
    <xsl:element name="ins" namespace="{$xhtmlns}">
      <xsl:attribute name="class">add</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <!--* de-only, en-only default rules *-->

  <xsl:template match="*" mode="de-only">
    <xsl:apply-templates select="."/>
  </xsl:template>
  <xsl:template match="*" mode="en-only">
    <xsl:apply-templates select="."/>
  </xsl:template>
</xsl:stylesheet>
