<!DOCTYPE xsl:stylesheet [
<!ENTITY lsquo  "&#x2018;" >
<!ENTITY rsquo  "&#x2019;" >
<!ENTITY ldquo  "&#x201C;" >
<!ENTITY rdquo  "&#x201D;" >
]>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:p5="http://www.tei-c.org/ns/1.0"
  xmlns:p5exx = "http://www.tei-c.org/ns/Examples"
  xmlns:rng="http://relaxng.org/ns/structure/1.0" 
  xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
  xmlns:sch="http://purl.oclc.org/dsdl/schematron"
  xmlns="http://www.w3.org/1999/xhtml"
>



 <!--* odd.xsl:  format a relatively simple TEI ODD file in HTML

     * Copyright (C) 2015 Black Mesa Technologies LLC

     * This program is free software: you can redistribute it and/or
     * modify it under the terms of the GNU General Public License as
     * published by the Free Software Foundation, either version 3 of
     * the License, or (at your option) any later version.

     * This program is distributed in the hope that it will be useful,
     * but WITHOUT ANY WARRANTY; without even the implied warranty of
     * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     * GNU General Public License for more details.

     * You should have received a copy of the GNU General Public
     * License along with this program, in the Licenses subdirectory,
     * in file GNU_GPL.

     * If the license was not provided, or is missing, see
     * <http://www.gnu.org/licenses/>.
     *-->

 <!--* no imports - this is not a top-level stylesheet, just a module *-->
 <!--* <xsl:import href="teilite.xsl"/> *-->
 <!--*  <xsl:import href="uikedocs.restricted.xsl"/> *-->

 <xsl:strip-space elements="p5:sequence p5:choice p5:content"/>

 <xsl:param name="siteroot"/>
 <xsl:param name="addedcss"><!--* addedcss, not altcss *-->
  <xsl:value-of select="concat($siteroot,'/lib/odd.css')"/>
 </xsl:param>

 <!--* content model layout:  horizontal (DTD syntax)
     * or vertical (list style)
     * or frege (Begriffsschrift style, SVG)
     *
     * Currently only horizontal is implemented.
     *-->
 <xsl:param name="cm-layout" select="'horizontal'"/>

 <xsl:variable name="nsXHTML"
	       select="'http://www.w3.org/1999/xhtml'"/>
 <xsl:variable name="nsTEI"
	       select="'http://www.tei-c.org/ns/1.0'" />
 <xsl:variable name="nsATMO"
	       select="'http://uyghur.ittc.ku.edu/2015/ns/0.1'" />
 <xsl:variable name="nsRNG"
	       select="'http://relaxng.org/ns/structure/1.0'"/>
 <xsl:variable name="nsXSI"
	       select="'http://www.w3.org/2001/XMLSchema-instance'" />
 <xsl:variable name="nsXSD"
	       select="'http://www.w3.org/2001/XMLSchema'" />

 <xsl:template match="p5:hi[@rend and contains(' blue red gray green ', @rend)]">
  <xsl:element name="span">
   <xsl:attribute name="class"><xsl:value-of select="@rend"/></xsl:attribute>
   <xsl:apply-templates/>
  </xsl:element>
 </xsl:template>

 <xsl:template match="p5:table/p5:row/p5:cell">
   <xsl:choose>
     <xsl:when test="@role='label' or ../@role = 'label'">
       <xsl:element name="th">
	 <xsl:apply-templates select="@rows|@cols"/>
         <xsl:call-template name="cross-walk-rendering"/>
	 <xsl:apply-templates/>
       </xsl:element>
     </xsl:when>
     <xsl:otherwise>
       <xsl:element name="td">
	 <xsl:apply-templates select="@rows|@cols"/>
         <xsl:call-template name="cross-walk-rendering"/>
	 <xsl:apply-templates/>
       </xsl:element>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <xsl:template name="cross-walk-rendering">
   <xsl:variable name="pos" 
		 select="1 + count(preceding-sibling::p5:cell)"/>
   <xsl:choose>
     <xsl:when test="@rend = 'lhs' or @rend = 'rhs'">
       <xsl:attribute name="class">
	 <xsl:value-of select="@rend"/>
       </xsl:attribute>
     </xsl:when>
     <xsl:when test="$pos = 1 or $pos = 3">
       <xsl:attribute name="class">
	 <xsl:value-of select="'lhs'"/>
       </xsl:attribute>
     </xsl:when>
     <xsl:when test="$pos = 2 or $pos = 4">
       <xsl:attribute name="class">
	 <xsl:value-of select="'rhs'"/>
       </xsl:attribute>
     </xsl:when>
     <xsl:otherwise>
       <xsl:message>Unexpected cell @rend value ...</xsl:message>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <xsl:template match="p5:gi[@scheme='TEI' or not(@scheme)]">
  <xsl:element name="tt">
    <xsl:element name="a">
      <xsl:attribute name="class">
	<xsl:value-of select="'giref'"/>
      </xsl:attribute>
      <xsl:attribute name="href">
	<xsl:value-of select="concat(
	  'http://www.tei-c.org/release/doc/tei-p5-doc/en/html/ref-',
	  string(.),
	  '.html')"/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:element>
 </xsl:template>

 <xsl:template match="p5:tag">
  <xsl:element name="tt">
    <!--* <xsl:text>&lt;</xsl:text> *-->
    <xsl:element name="a">
      <xsl:attribute name="class">
	<xsl:value-of select="'giref'"/>
      </xsl:attribute>
      <xsl:attribute name="href">
	<xsl:value-of select="concat(
	  'http://www.tei-c.org/release/doc/tei-p5-doc/en/html/ref-',
	  substring-before(concat(.,' '), ' '),
	  '.html')"/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
    <!--* <xsl:text>&gt;</xsl:text> *-->
  </xsl:element>
 </xsl:template>

 <xsl:template match="p5:equiv">
  <xsl:element name="p">
    <xsl:text>Equivalent to: </xsl:text>
    <!--* <xsl:value-of select="@ns"/> *-->
    <xsl:value-of select="@name"/>
    <xsl:text>.</xsl:text>
  </xsl:element>
 </xsl:template>

 <!--* P5 ODD elements *-->
 <!--* Much of this borrowed from code published by the
     * Institut für Deutsche Sprache (IDS), Mannheim
     * and open-sourced by IDS.
     * Some of it, however, is new.
     *-->
 <xsl:template match="p5:schemaSpec">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">schemaSpec</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">head</xsl:attribute>
       <xsl:text>Spec fragment </xsl:text>
       <xsl:element name="span">
	 <xsl:attribute name="class">specid</xsl:attribute>
	 <xsl:value-of select="@ident"/>
       </xsl:element>
     </xsl:element>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:specGrp">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">specGrp</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">head</xsl:attribute>
       <xsl:text>Spec fragment </xsl:text>
       <xsl:element name="span">
	 <xsl:attribute name="class">specfragid</xsl:attribute>
	 <xsl:value-of select="@xml:id"/>
       </xsl:element>
     </xsl:element>
     <xsl:apply-templates/>
     <xsl:element name="div">
       <xsl:attribute name="class">usedby</xsl:attribute>
       <xsl:variable name="this" select="concat('#',@xml:id)"/>
       <xsl:variable name="callers" 
		     select="//p5:specGrpRef[@target=$this]
			     /ancestor::p5:*
			     [self::p5:specGrp or self::p5:schemaSpec]
			     [1]"/>
       <xsl:choose>
	 <xsl:when test="not($callers)">
	   <xsl:element name="p">This spec fragment is not used.</xsl:element>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:element name="p">
	     <xsl:text>This spec fragment is referred to by </xsl:text>
	     <xsl:for-each select="$callers">
	       <xsl:choose>
		 <xsl:when test="self::p5:schemaSpec">
		   <xsl:text>top-level schema fragment </xsl:text>
		 </xsl:when>
		 <xsl:when test="self::p5:specGrp">
		   <xsl:text> spec fragment </xsl:text>
		 </xsl:when>
		 <xsl:otherwise>
		 </xsl:otherwise>		 
	       </xsl:choose>
	       <xsl:element name="span">
		 <xsl:attribute name="class">specfragid</xsl:attribute>
		 <xsl:element name="a">
		   <xsl:variable name="caller-id">
		     <xsl:call-template name="get-id"/>
		   </xsl:variable>
		   <xsl:attribute name="href">
		     <xsl:text>#</xsl:text>
		     <xsl:value-of select="$caller-id"/>
		     <!--* <xsl:value-of select="concat('#',@xml:id)"/> *-->
		   </xsl:attribute>
		   <xsl:choose>
		     <xsl:when test="self::p5:schemaSpec">
		       <xsl:value-of select="@ident"/>
		     </xsl:when>
		     <xsl:otherwise>
		       <xsl:value-of select="$caller-id"/>
		     </xsl:otherwise>
		   </xsl:choose>
		 </xsl:element>
	       </xsl:element>
	       <xsl:choose>
		 <xsl:when test="count($callers) = 1"/>
		 <xsl:when test="position() = last()"/>
		 <xsl:when test="count($callers) > 2
				 and position() &lt; last() - 1">
		   <xsl:text>, </xsl:text>
		 </xsl:when>
		 <xsl:when test="count($callers) > 2 and position()
				 = last() - 1">
		   <xsl:text>, and </xsl:text>
		 </xsl:when>
		 <xsl:when test="count($callers) = 2 and position()
				 = 1">
		   <xsl:text> and </xsl:text>
		 </xsl:when>
		 <xsl:otherwise>
		   <xsl:text> and </xsl:text>
		 </xsl:otherwise>
	       </xsl:choose>
	     </xsl:for-each>
	     <xsl:text>. </xsl:text>
	   </xsl:element>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:specGrpRef">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">specGrpRef</xsl:attribute>
     <xsl:text>Include spec fragment </xsl:text>
     <xsl:element name="span">
       <xsl:attribute name="class">specfragid</xsl:attribute>
       <xsl:element name="a">
	 <xsl:attribute name="href">
	   <xsl:value-of select="@target"/>
	 </xsl:attribute>
	 <xsl:value-of select="substring-after(@target,'#')"/>
       </xsl:element>
     </xsl:element>
     <xsl:if test="not(substring-after(@target,'#') = //@xml:id)">
       <xsl:text> (!! not found !!)</xsl:text>
     </xsl:if>
     <xsl:text>.</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:moduleRef">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">moduleRef</xsl:attribute>
     <xsl:text>Include module </xsl:text>
     <xsl:value-of select="@key"/>
     <xsl:text>.</xsl:text>
     <xsl:apply-templates select="@except | @include"/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:moduleRef/@except" name="module-exceptions">
   <xsl:param name="lGi" select="normalize-space(.)"/>
   <xsl:choose>
     <xsl:when test="contains($lGi,' ')">
       <!--* normal case:  we have two or more names to handle *-->
       <xsl:variable name="gi" select="substring-before($lGi,' ')"/>
       <xsl:call-template name="drop-element">
	 <xsl:with-param name="gi" select="$gi"/>
       </xsl:call-template>
       <xsl:call-template name="module-exceptions">
	 <xsl:with-param name="lGi" select="substring-after($lGi,' ')"/>
       </xsl:call-template>
     </xsl:when>
     <xsl:when test="$lGi = ''">
       <!--* unexpected case:  we have no names at all *-->
     </xsl:when>
     <xsl:otherwise>
       <!--* Normal termination case:  one name left *-->
       <xsl:call-template name="drop-element">
	 <xsl:with-param name="gi" select="$lGi"/>
       </xsl:call-template>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>
 
 <xsl:template match="p5:moduleRef/@include" name="module-inclusions">
   <xsl:param name="lGi" select="normalize-space(.)"/>
   <xsl:choose>
     <xsl:when test="contains($lGi,' ')">
       <!--* normal case:  we have two or more names to handle *-->
       <xsl:variable name="gi" select="substring-before($lGi,' ')"/>
       <xsl:call-template name="include-element">
	 <xsl:with-param name="gi" select="$gi"/>
       </xsl:call-template>
       <xsl:call-template name="module-inclusions">
	 <xsl:with-param name="lGi" select="substring-after($lGi,' ')"/>
       </xsl:call-template>
     </xsl:when>
     <xsl:when test="$lGi = ''">
       <!--* unexpected case:  we have no names at all *-->
     </xsl:when>
     <xsl:otherwise>
       <!--* Normal termination case:  one name left *-->
       <xsl:call-template name="include-element">
	 <xsl:with-param name="gi" select="$lGi"/>
       </xsl:call-template>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <xsl:template match="p5:macroSpec">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">macroSpec</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">head</xsl:attribute>
       <xsl:text>Define macro </xsl:text>
       <xsl:element name="code">
	 <xsl:value-of select="@ident"/>
       </xsl:element>
       <xsl:text>.</xsl:text>
     </xsl:element>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec[@mode='delete']">
   <xsl:param name="gi" select="@ident"/>
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">elemDrop</xsl:attribute>
     <xsl:call-template name="drop-element">
       <xsl:with-param name="gi" select="$gi"/>
     </xsl:call-template>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template name="drop-element">
   <xsl:param name="gi" select="@ident"/>
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">elemDrop</xsl:attribute>
     <xsl:text>Drop element </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="$gi"/>
     </xsl:element>
     <xsl:text>.</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template name="include-element">
   <xsl:param name="gi" select="@ident"/>
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">elemIncl</xsl:attribute>
     <xsl:text>Include element </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="$gi"/>
     </xsl:element>
     <xsl:text>.</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec[@mode='change']">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">elemChange</xsl:attribute>
     <xsl:text>Change element </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="@ident"/>
     </xsl:element>
     <xsl:text>:  </xsl:text>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec[@mode='change']/p5:altIdent">
   <xsl:element name="span">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">altIdent</xsl:attribute>
     <xsl:text>rename as &ldquo;</xsl:text>
     <xsl:element name="code">
       <xsl:apply-templates/>
     </xsl:element>
     <xsl:text>&rdquo;.</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec[@mode='add' or not(@mode)]/p5:altIdent">
   <xsl:message>alternative reality ... <xsl:value-of select="."/></xsl:message>
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Identifier</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec[@mode='add' or not(@mode)]">
   <xsl:variable name="ns-prefix">
     <xsl:call-template name="ns-prefix-from-ns">
       <xsl:with-param name="ns-name" select="@ns"/>
     </xsl:call-template>
   </xsl:variable>
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">elemSpec</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">head</xsl:attribute>
       <xsl:text>&lt;</xsl:text>
       <xsl:choose>
	 <xsl:when test="$ns-prefix = ''">
	   <xsl:value-of select="@ident"/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:value-of select="concat($ns-prefix, ':', @ident)"/>
	 </xsl:otherwise>
       </xsl:choose>
       <xsl:text>&gt; element (new)</xsl:text>
     </xsl:element>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:desc">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>&#xA0;</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:elementSpec/p5:gloss | p5:macroSpec/p5:gloss">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>i.e.</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:classes[*]">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Classes</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:classes/p5:memberOf">
   <xsl:element name="div">
     <xsl:element name="span">
       <xsl:attribute name="class">comment</xsl:attribute>
       <xsl:choose>
	 <xsl:when test="@mode='add'">
	   <xsl:text>(add) </xsl:text>
	 </xsl:when>
	 <xsl:when test="@mode='delete'">
	   <xsl:text>(drop) </xsl:text>
	 </xsl:when>
	 <xsl:when test="@mode='change'">
	   <xsl:text>(change) </xsl:text>
	 </xsl:when>
       </xsl:choose>
     </xsl:element>
     <xsl:element name="a">
       <xsl:attribute name="href">
	 <xsl:text>#</xsl:text>
	 <xsl:value-of select="@key"/>
       </xsl:attribute>
       <xsl:value-of select="@key"/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:content">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:choose>
	 <xsl:when test="parent::p5:elementSpec/@mode='change'">
	   <xsl:text>Change contents to</xsl:text>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:text>Contains</xsl:text>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:content[rng:empty]">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:choose>
	 <xsl:when test="parent::p5:elementSpec/@mode='change'">
	   <xsl:text>Change contents to</xsl:text>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:text>Contains</xsl:text>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:text>Empty.</xsl:text>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:constraintSpec[@scheme='isoschematron']">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Schematron constraint</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:constraintSpec/p5:constraint">
   <xsl:apply-templates/>
 </xsl:template>

 <!--* suppress Schematron ns elements for now. *-->
 <xsl:template match="sch:ns"/>

 <xsl:template match="p5:constraintSpec//sch:report">
   <xsl:element name="div">
     <xsl:text>When the condition </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="@test"/>
     </xsl:element>
     <xsl:text> holds, report: &ldquo;</xsl:text>
     <!--* <xsl:apply-templates/> *-->
     <xsl:value-of select="normalize-space(.)"/>
     <xsl:text>&rdquo;</xsl:text>
   </xsl:element>
 </xsl:template>
 <xsl:template match="p5:constraintSpec//sch:assert">
   <xsl:element name="div">
     <xsl:text>If the assertion </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="@test"/>
     </xsl:element>
     <xsl:text> fails, report: &ldquo;</xsl:text>
     <!--* <xsl:apply-templates/> *-->
     <xsl:value-of select="normalize-space(.)"/>
     <xsl:text>&rdquo;</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template match="rng:text">
   <xsl:text>#PCDATA</xsl:text>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template match="p5:content/rng:text[not(preceding-sibling::*) 
		      and not(following-sibling::*)]">
   <xsl:text>character data</xsl:text>
 </xsl:template>

 <xsl:template match="rng:*/text()"/>

 <xsl:template match="rng:group | rng:choice | rng:interleave
		      | p5:alternate | p5:sequence">
   <xsl:text>(</xsl:text>
   <xsl:apply-templates/>
   <xsl:text>)</xsl:text>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template match="rng:oneOrMore">
   <xsl:apply-templates/>
   <xsl:text>+</xsl:text>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template match="rng:zeroOrMore">
   <xsl:apply-templates/>
   <xsl:text>*</xsl:text>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template match="rng:optional">
   <xsl:apply-templates/>
   <xsl:text>?</xsl:text>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template match="rng:ref">
   <xsl:variable name="ns-prefix">     
     <xsl:call-template name="ns-prefix-from-ns">
       <xsl:with-param name="ns-name" select="@ns"/>
     </xsl:call-template>
   </xsl:variable>
   <xsl:variable name="gi-prefix">
     <xsl:choose>
       <xsl:when test="ancestor::p5:content/@autoPrefix = 'false'">
	 <!--* this is a hacky way of dealing with the prefixing
	     * in ODDs.  It assumes the prefix used is a conventional
	     * namespace prefix. In our ODDs this is so.
	     *-->
	 <xsl:value-of select="concat(substring-before(@name,'_'), ':')"/>
       </xsl:when>
       <xsl:when test="not(@ns) ">
	 <xsl:value-of select="''"/>
       </xsl:when>
       <xsl:when test="$ns-prefix = '' ">
	 <xsl:value-of select="''"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="concat($ns-prefix, ':')"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:variable name="odd_prefix">
     <xsl:choose>
       <xsl:when test="ancestor::p5:content/@autoPrefix='false'">
	 <xsl:value-of select="''"/>
       </xsl:when>
       <xsl:when test="not(@prefix) or @prefix = ''">
	 <xsl:value-of select="''"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="concat(@prefix, '_')"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:choose>
     <xsl:when test="starts-with(@name,'model.')">
       <xsl:value-of select="concat('%',$odd_prefix,@name,';')"/>
     </xsl:when>
     <xsl:when test="starts-with(@name,'class.')">
       <xsl:value-of select="concat('%',$odd_prefix,@name,';')"/>
     </xsl:when>
     <xsl:when test="starts-with(@name,'att.')">
       <xsl:value-of select="concat('%',odd_prefix,@name,';')"/>
     </xsl:when>     
       <xsl:when test="ancestor::p5:content/@autoPrefix='false'">
	 <xsl:value-of select="concat($gi-prefix,substring-after(@name,'_'))"/>
       </xsl:when>
     <xsl:otherwise>
       <xsl:value-of select="concat($gi-prefix,@name)"/>
     </xsl:otherwise>
   </xsl:choose>
   <xsl:call-template name="rng-separator"/>
 </xsl:template>

 <xsl:template name="rng-separator">
   <xsl:choose>
     <xsl:when test="not(following-sibling::*)"/>
     <xsl:when test="parent::rng:choice">
       <xsl:text> | </xsl:text>
     </xsl:when>
     <xsl:when test="parent::rng:group">
       <xsl:text>, </xsl:text>
     </xsl:when>
     <xsl:when test="parent::rng:interleave">
       <xsl:text> &amp; </xsl:text>
     </xsl:when>
     <xsl:when test="parent::p5:alternate">
       <xsl:text> | </xsl:text>
     </xsl:when>
     <xsl:when test="parent::p5:sequence">
       <xsl:text>, </xsl:text>
     </xsl:when>
     <xsl:otherwise>
       <xsl:value-of select="@name"/>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <xsl:template match="xsd:date">
   <xsl:text>a date in yyyy-mm-dd form (an XSD date value)</xsl:text>
 </xsl:template>

 <xsl:template match="rng:data">
   <xsl:text>an XSD </xsl:text>
   <xsl:element name="code">
     <xsl:value-of select="@type"/>
   </xsl:element>
   <xsl:text> value</xsl:text>
 </xsl:template>

 <xsl:template match="rng:list[rng:data]">
   <xsl:text>a list of XSD </xsl:text>
   <xsl:element name="code">
     <xsl:value-of select="@type"/>
   </xsl:element>
   <xsl:text> values</xsl:text>
 </xsl:template>

 <xsl:template match="p5:attList">
   <xsl:element name="div">
     <xsl:attribute name="class">attlist</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">lvpair</xsl:attribute>
       <xsl:element name="div">
	 <xsl:attribute name="class">label</xsl:attribute>
	 <xsl:text>Attributes</xsl:text>
       </xsl:element>
       <xsl:element name="div">
	 <xsl:attribute name="class">value</xsl:attribute>
	 <xsl:text>&#xA0;</xsl:text>
       </xsl:element>
     </xsl:element>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:attRef">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair attref</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label attname</xsl:attribute>
       <xsl:value-of select="@name"/>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value attref</xsl:attribute>
       <xsl:element name="span">
	 <xsl:attribute name="class">comment</xsl:attribute>
	 <xsl:text>from attribute class </xsl:text>
	 <xsl:element name="span" namespace="{$nsXHTML}">
	   <xsl:attribute name="class">
	     <xsl:text>attref</xsl:text>
	   </xsl:attribute>
	   <xsl:value-of select="@class"/>
	 </xsl:element>
	 <xsl:text>.</xsl:text>
       </xsl:element>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:attDef">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair attdef</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label attname</xsl:attribute>
       <xsl:value-of select="@ident"/>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:element name="span">
	 <xsl:attribute name="class">comment</xsl:attribute>
	 <xsl:choose>
	   <xsl:when test="@mode='add'">
	     <xsl:text>(add this attribute) </xsl:text>
	   </xsl:when>
	   <xsl:when test="@mode='delete'">
	     <xsl:text>(drop this attribute) </xsl:text>
	   </xsl:when>
	   <xsl:when test="@mode='change'">
	     <xsl:text>(change this attribute) </xsl:text>
	   </xsl:when>
	   <xsl:when test="@mode='replace'">
	     <xsl:text>(redefine this attribute) </xsl:text>
	   </xsl:when>
	 </xsl:choose>
       </xsl:element>
       <xsl:apply-templates/>
       <xsl:apply-templates select="@usage"/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:attDef[@mode='delete']">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair attdef</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label attname</xsl:attribute>
       <xsl:value-of select="@ident"/>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:element name="i">(delete this attribute)</xsl:element>
     </xsl:element>
   </xsl:element>
 </xsl:template>
 
 <xsl:template match="p5:attDef/@usage">
   <xsl:element name="div">
     <xsl:attribute name="class">attusage</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">lvpair</xsl:attribute>
       <xsl:element name="div">
	 <xsl:attribute name="class">label</xsl:attribute>
	 <xsl:text>Usage</xsl:text>       
       </xsl:element>
       <xsl:element name="div">
	 <xsl:attribute name="class">value</xsl:attribute>
	 <xsl:choose>
	   <xsl:when test=". = 'opt'">
	     <xsl:text>optional</xsl:text>
	   </xsl:when>
	   <xsl:when test=". = 'req'">
	     <xsl:text>required</xsl:text>
	   </xsl:when>
	   <xsl:when test=". = 'rec'">
	     <xsl:text>recommended</xsl:text>
	   </xsl:when>
	 </xsl:choose>
       </xsl:element>
     </xsl:element>
   </xsl:element>   
 </xsl:template>

 <xsl:template match="p5:attDef/p5:desc">
   <xsl:element name="div">
     <xsl:attribute name="class">attdesc</xsl:attribute>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:datatype">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Type</xsl:text>       
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:defaultVal">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Default value</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:valList">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Values</xsl:text>       
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:choose>
	 <xsl:when test="@type='open'">
	   <xsl:text>(open list)</xsl:text>
	 </xsl:when>
	 <xsl:when test="@type='closed'">
	   <xsl:text>(closed list)</xsl:text>
	 </xsl:when>
       </xsl:choose>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:valItem">
   <xsl:element name="div">
     <xsl:attribute name="class">lvpair valItem</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
	 <xsl:element name="code">
	   <xsl:value-of select="@ident"/>
	 </xsl:element>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">value</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:valItem/p5:gloss">
   <xsl:element name="span">
     <xsl:attribute name="class">valueGloss</xsl:attribute>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:valItem/p5:desc">
   <xsl:element name="span">
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:exemplum">
   <xsl:element name="div">
     <xsl:attribute name="class">exemplum</xsl:attribute>
     <xsl:element name="div">
       <xsl:attribute name="class">label</xsl:attribute>
       <xsl:text>Example:</xsl:text>
     </xsl:element>
     <xsl:element name="div">
       <xsl:attribute name="class">example</xsl:attribute>
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5exx:egXML">
   <xsl:element name="pre">
     <xsl:attribute name="class">egXML</xsl:attribute>
     <xsl:apply-templates mode="egXML"/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:remarks">
   <xsl:element name="div">
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:exemplum/p5:eg">
   <xsl:element name="pre">
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>
 <xsl:template match="p5:eg[not(parent::exemplum)]">
   <xsl:element name="div">
     <xsl:attribute name="class">free-example</xsl:attribute>
       <xsl:element name="pre">
       <xsl:apply-templates/>
     </xsl:element>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:listRef"/>

 <xsl:template match="p5:classSpec[@mode='change']">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">classChange</xsl:attribute>
     <xsl:text>Change class </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="@ident"/>
     </xsl:element>
     <xsl:text>.</xsl:text>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="p5:classSpec[@mode='add' or not(@mode)]">
   <xsl:element name="div">
     <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
     <xsl:attribute name="class">classAdd</xsl:attribute>
     <xsl:text>Add class </xsl:text>
     <xsl:element name="code">
       <xsl:value-of select="@ident"/>
     </xsl:element>
     <xsl:text>.</xsl:text>
     <xsl:apply-templates/>
   </xsl:element>
 </xsl:template>

 <!--* simple sequence *-->
 <!--*
 <xsl:template match="p5:sequence 
		      [(not(@minOccurs) or @minOccurs = 1) 
		      and (not(@maxOccurs) or @maxOccurs = 1) ]">
		      </xsl:template>
		      *-->

 <!--* sequence with occurrence indicator *-->
 <!--*
 <xsl:template match="p5:sequence
		      [not(@minOccurs = 1)
		      or not(@maxOccurs= 1)) ]"
	       name="sequence">
   
   <xsl:element name="div">
     <xsl:call-template name="occurrence-indicator"/>
     <xsl:call-template name="sequence"/>
   </xsl:element>
   </xsl:template>
   *-->

 <xsl:template name="modelgroup"
	       match="p5:sequence | p5:choice">
   <xsl:variable name="min">
     <xsl:choose>
       <xsl:when test="not(@minOccurs)">
	 <xsl:value-of select="1"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@minOccurs"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:variable name="max">
     <xsl:choose>
       <xsl:when test="not(@maxOccurs)">
	 <xsl:value-of select="1"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@maxOccurs"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   
   <xsl:choose>
     <xsl:when test="$cm-layout = 'horizontal'">
       <xsl:choose>
	 <xsl:when test="count(*) &lt; 2 and $min = 1 and $max = 1
			 and not(../self::p5:content)">
	   <xsl:apply-templates/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:text>(</xsl:text>
	   <xsl:apply-templates/>
	   <xsl:text>)</xsl:text>
	   <xsl:call-template name="occ-suffix">
	     <xsl:with-param name="min" select="$min"/>
	     <xsl:with-param name="max" select="$max"/>
	   </xsl:call-template>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:when>
     <xsl:when test="$cm-layout = 'vertical'">
       <xsl:element name="div">
	 <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
	 <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
	 <xsl:value-of select="local-name()"/>
	 <xsl:text> of:</xsl:text>
	 <xsl:apply-templates/>
       </xsl:element>
     </xsl:when>
     <xsl:when test="$cm-layout = 'frege'">
     </xsl:when>
   </xsl:choose>
 </xsl:template>

 <xsl:template match="p5:elementRef | p5:classRef">
   <xsl:variable name="min">
     <xsl:choose>
       <xsl:when test="not(@minOccurs)">
	 <xsl:value-of select="1"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@minOccurs"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:variable name="max">
     <xsl:choose>
       <xsl:when test="not(@maxOccurs)">
	 <xsl:value-of select="1"/>
       </xsl:when>
       <xsl:otherwise>
	 <xsl:value-of select="@maxOccurs"/>
       </xsl:otherwise>
     </xsl:choose>
   </xsl:variable>
   <xsl:choose>
     <xsl:when test="$cm-layout = 'horizontal'">
       <xsl:element name="span" namespace="{$nsXHTML}">
	 <xsl:attribute name="class">
	   <xsl:value-of select="local-name()"/>
	 </xsl:attribute>
	 <xsl:if test="self::p5:classRef">
	   <xsl:text>%</xsl:text>
	 </xsl:if>
	 <xsl:value-of select="@key"/>
	 <xsl:if test="self::p5:classRef">
	   <xsl:text>;</xsl:text>
	 </xsl:if>
       </xsl:element>
       <xsl:call-template name="occ-suffix">
	 <xsl:with-param name="min" select="$min"/>
	 <xsl:with-param name="max" select="$max"/>
       </xsl:call-template>
       <xsl:call-template name="separator"/>
     </xsl:when>
     <xsl:when test="$cm-layout = 'vertical'">
       <xsl:element name="div">
	 <xsl:attribute name="id"><xsl:call-template name="get-id"/></xsl:attribute>
	 <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
	 <xsl:call-template name="occurrence-indicator"/>
	 <xsl:text>element </xsl:text>
	 <xsl:value-of select="@key"/>
	 <xsl:apply-templates/>
       </xsl:element>
     </xsl:when>
     <xsl:when test="$cm-layout = 'frege'">
     </xsl:when>
   </xsl:choose>
 </xsl:template>
 
 <xsl:template match="p5:macroRef">
   <xsl:element name="span" namespace="{$nsXHTML}">
     <xsl:attribute name="class">
       <xsl:value-of select="local-name()"/>
     </xsl:attribute>
     <xsl:value-of select="@key"/>
   </xsl:element>
 </xsl:template>

 <xsl:template match="*" mode="egXML">
   <xsl:element name="span" namespace="{$nsXHTML}">
     <xsl:attribute name="class">
       <xsl:value-of select="'starttag'"/>
     </xsl:attribute>
     
     <xsl:text>&lt;</xsl:text>
     <xsl:value-of select="name()"/>
     <xsl:for-each select="@*">
       <xsl:value-of select="concat(' ', name(), '=', '&quot;', string(.), '&quot;')"/>
     </xsl:for-each>
     <xsl:text>&gt;</xsl:text>
     
     <xsl:apply-templates mode="egXML"/>
     
     <xsl:text>&lt;/</xsl:text>
     <xsl:value-of select="name()"/>            
     <xsl:text>&gt;</xsl:text>
   </xsl:element>
 </xsl:template>

 <xsl:template match="text()" mode="egXML">
   <xsl:copy-of select="."/>
 </xsl:template>
 

 <xsl:template name="occurrence-indicator">
 </xsl:template>

 <!--* occ-suffix emits suffix occurrence operators *-->
 <xsl:template name="occ-suffix">
   <xsl:param name="min"/><!--* required="yes"/> *-->
   <xsl:param name="max"/><!--* required="yes"/> *-->
   <xsl:choose>
     <xsl:when test="$min = 1 and $max = 1">
     </xsl:when>
     <xsl:when test="$min = 0 and $max = 1">
       <xsl:text>?</xsl:text>
     </xsl:when>
     <xsl:when test="$min = 1 and ($max = 'unbounded' or $max = '*'
		     or $max = '+')">
       <xsl:text>+</xsl:text>
     </xsl:when>
     <xsl:when test="$min = 0 and ($max = 'unbounded' or $max = '*'
		     or $max = '+')">
       <xsl:text>*</xsl:text>
     </xsl:when>
     <xsl:otherwise>
       <xsl:value-of select="concat('{', $min, ', ', $max, '}')"/>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <!--* separator:  comma, or-bar, ampersand in horizontal display
     of content models *-->
 <xsl:template name="separator">
   <xsl:choose>
     <xsl:when test="not(following-sibling::*)"/>
     <xsl:when test="parent::p5:sequence">
       <xsl:text>, </xsl:text>
     </xsl:when>
     <xsl:when test="parent::p5:choice">
       <xsl:text> |&#xA0;</xsl:text>
     </xsl:when>
     <xsl:when test="parent::p5:all">
       <xsl:text> &amp;&#xA0;</xsl:text>
     </xsl:when>
   </xsl:choose>
 </xsl:template>
 
 <xsl:template name="get-id">
  <xsl:param name="e" select="."/>
  <xsl:choose>
   <xsl:when test="$e/self::p5:classSpec/@ident"><xsl:value-of select="$e/@ident"/></xsl:when>
   <xsl:when test="$e/@xml:id"><xsl:value-of select="$e/@xml:id"/></xsl:when>
   <xsl:otherwise><xsl:value-of select="generate-id($e)"/></xsl:otherwise>
  </xsl:choose>  
 </xsl:template>

 <xsl:template name="ns-prefix-from-ns">
   <xsl:param name="ns-name" select="@ns"/>
   <!--* hard-coding these is almost certainly a horrible idea *-->
   <xsl:variable name="ns-prefix-0"
		 select="name(ancestor-or-self::*[1]/namespace::*[. = $ns-name])"/>
   <xsl:choose>
     <xsl:when test="$ns-prefix-0 != ''">
       <xsl:value-of select="$ns-prefix-0"/>
     </xsl:when>
     <xsl:when test="$ns-name = $nsTEI">
       <xsl:text>tei</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsATMO">
       <xsl:text>atmo</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXHTML">
       <xsl:text>xh</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsRNG">
       <xsl:text>rng</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXSI">
       <xsl:text>xsi</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXSD">
       <xsl:text>xsd</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = '' ">
       <xsl:text></xsl:text>
     </xsl:when>
     <xsl:otherwise>
       <xsl:text>NSPREFIX</xsl:text>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>

 <!--* for reasons not yet clear, including the following
     template causes an XSLT parse error in Firefox.
     We seem not to call it, so I'm commenting it out
     until it's needed again.
     *-->
   <!--*
 
 <xsl:template name="ns-from-ns-prefix">
   <xsl:param name="ns-prefix" required="yes"/>
   <\!\-\-* hard-coding these is almost certainly a horrible idea *\-\-\>
   <xsl:text>NSP-temphack</xsl:text>
   <xsl:variable name="ns-prefix-0"
		 select="ancestor-or-self::*[1]/namespace::*[name() = $ns-prefix]"/>
   <xsl:choose>
     <xsl:when test="$ns-prefix-0 != ''">
       <xsl:value-of select="$ns-prefix-0"/>
     </xsl:when>
     <xsl:when test="$ns-name = $nsTEI">
       <xsl:text>tei</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsATMO">
       <xsl:text>atmo</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXHTML">
       <xsl:text>xh</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsRNG">
       <xsl:text>rng</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXSI">
       <xsl:text>xsi</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = $nsXSD">
       <xsl:text>xsd</xsl:text>
     </xsl:when>
     <xsl:when test="$ns-name = '' ">
       <xsl:text></xsl:text>
     </xsl:when>
     <xsl:otherwise>
       <xsl:text>NSPREFIX</xsl:text>
     </xsl:otherwise>
   </xsl:choose>
 </xsl:template>
   *-->

</xsl:stylesheet>
<!-- Keep this comment at the end of the file
Local variables:
mode: xml
sgml-default-dtd-file:"/Library/SGML/Public/Emacs/xslt.ced"
sgml-omittag:t
sgml-shorttag:t
sgml-indent-data:t
sgml-indent-step:1
End:
-->
