<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:p5="http://www.tei-c.org/ns/1.0"
    xmlns:xh="http://www.w3.org/1999/xhtml"
    xmlns="http://www.w3.org/1999/xhtml"
    exclude-result-prefixes="xs p5 xh"
    version="1.0">
    
    <!--* p5_to_html.xsl:  translate TEI P5 to (X)HTML (generic).
        * Intended for import by other stylesheets which impose
        * structure on the XHTML output, but should work standalone.
        *
        * Copyright (C) 2016 Black Mesa Technologies LLC
     
        * This program is free software: you can redistribute it and/or
        * modify it under the terms of the GNU General Public License as
        * published by the Free Software Foundation, either version 3 of
        * the License, or (at your option) any later version.
     
        * This program is distributed in the hope that it will be useful,
        * but WITHOUT ANY WARRANTY; without even the implied warranty of
        * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        * GNU General Public License for more details.
     
        * You should have received a copy of the GNU General Public
        * License along with this program, in the Licenses subdirectory,
        * in file GNU_GPL.
     
        * If the license was not provided, or is missing, see
        * <http://www.gnu.org/licenses/>.
        *-->
    
    <!--* 0 setup *-->
    <!--* 1 Default style is red not-styled-yet display *-->
    <!--* 2 TEI and overall document structure *-->
    <!--* 3 teiHeader *-->
    <!--* 4 text, divs *-->
    <!--* 5 front *-->
    <!--* 6 back *-->
    <!--* 7 paragraphs etc. *-->
    <!--* 8 phrase-level markup *-->
    
    <!--* 0 setup *-->

    <xsl:include href="odd.xsl"/>

    <!--* num-sec:  number sections?  yes or no *-->
    <xsl:param name="num-sec" select=" 'no' "/>

    <xsl:variable name="xhtmlns" select="'http://www.w3.org/1999/xhtml'"/>


    <xsl:template name="inject-css-link"/>
   
    <!--* 1 Default style is red not-styled-yet display *-->
    <xsl:template match="*" name="unknown-element">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'unknown'"/>
            </xsl:attribute>
            <xsl:attribute name="style">
                <xsl:value-of select="'color: red; margin-left: 1em;'"/>
            </xsl:attribute>
            
            <xsl:text>&lt;</xsl:text>
            <xsl:value-of select="name()"/>
            <xsl:for-each select="@*">
                <xsl:value-of select="concat(' ', name(), '=', '&quot;', string(.), '&quot;')"/>
            </xsl:for-each>
            <xsl:text>&gt;</xsl:text>
            
            <xsl:apply-templates/>
                        
            <xsl:text>&lt;/</xsl:text>
            <xsl:value-of select="name()"/>            
            <xsl:text>&gt;</xsl:text>
        </xsl:element>
    </xsl:template>
    
    <!--* 2 TEI and overall document structure *-->
    <xsl:template match="/">
        <xsl:element name="html" namespace="{$xhtmlns}">
            <xsl:element name="head" namespace="{$xhtmlns}">
                <xsl:element name="title" namespace="{$xhtmlns}">
                    <xsl:apply-templates 
                        select="/p5:TEI/p5:teiHeader/p5:fileDesc/p5:titleStmt/p5:title[1]"
                        mode="htmlhead"/>
                </xsl:element>
		<xsl:call-template name="inject-css-link"/>
            </xsl:element>
            <xsl:element name="body" namespace="{$xhtmlns}">
                <xsl:apply-templates/>
		<xsl:if test="//p5:note[@place='foot']">
		  <xsl:element name="div" namespace="{$xhtmlns}">
                    <xsl:attribute name="class">end-notes</xsl:attribute>
                    <xsl:apply-templates select="//p5:note[@place='foot']"
					 mode="notes"/>
		  </xsl:element>
		</xsl:if>
            </xsl:element>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:title" mode="htmlhead">
        <xsl:value-of select="string(.)"/>
    </xsl:template>
    
    <!--* simple blocks:  div with class attribute *-->
    <xsl:template match="p5:TEI|p5:text|p5:front|p5:body|p5:back|p5:titlePage">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    
    <!--* 3 teiHeader *-->
    <xsl:template match="p5:teiHeader"/>
    
    <!--* 4 text, divs *-->
    
    <xsl:template match="p5:div">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat('div', count(ancestor-or-self::p5:div))"/>
            </xsl:attribute>
            <xsl:attribute name="id">
                <xsl:choose>
                    <xsl:when test="@xml:id">
                        <xsl:value-of select="@xml:id"/>
                    </xsl:when>
                    <xsl:when test="@id">
                        <xsl:value-of select="@id"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="generate-id(.)"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
	    <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:div/p5:head">
      <xsl:variable name="classname"
		    select="concat('head', count(ancestor::p5:div))"/>
      <xsl:variable name="Hn"
		    select="concat('h', count(ancestor::p5:div) + 2)"/>
        <xsl:element name="{$Hn}" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="$classname"/></xsl:attribute>

	    <xsl:if test="$num-sec = 'yes'">
	      <xsl:choose>
		<xsl:when test="ancestor::p5:front">
		</xsl:when>
		<xsl:when test="ancestor::p5:body">
		  <xsl:number count="p5:div[p5:head]"
			      level="multiple"
			      from="p5:body"
			      format="1.2.3.4"/>
		  <text>&#xA0;</text>
		</xsl:when>
		<xsl:when test="ancestor::p5:back">
		  <xsl:number count="p5:div[p5:head]"
			      level="multiple"
			      from="p5:back"
			      format="A.2.3.4"/>
		  <text>&#xA0;</text>
		</xsl:when>
	      </xsl:choose>
	    </xsl:if>	    
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <!--*
    <xsl:template match="p5:body/p5:div/p5:head | p5:back/p5:div/p5:head">
        <xsl:element name="h3" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="'head1'"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>   
    
    <xsl:template match="p5:body/p5:div/p5:div/p5:head">
        <xsl:element name="h4" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="'head2'"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>   
    
    <xsl:template match="p5:body/p5:div/p5:div/p5:div/p5:head">
        <xsl:element name="h5" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="'head3'"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:body/p5:div/p5:div/p5:div/p5:div/p5:head">
        <xsl:element name="h5" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="'head3'"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
	</xsl:template>
	*-->
    
    <!--* 5 front *-->
    <xsl:template match="p5:docTitle">
        <xsl:element name="h1" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:docTitle/p5:titlePart">
        <xsl:if test="preceding-sibling::p5:titlePart">
            <xsl:element name="br" namespace="{$xhtmlns}"/>                
        </xsl:if>
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:docAuthor|p5:docDate">
        <xsl:element name="h2" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:titlePage/p5:titlePart">
        <xsl:element name="h2" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:front/p5:div">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' front')"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:front/p5:div/p5:head">
        <xsl:element name="h3" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' front')"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
        
    <xsl:template match="p5:divGen[@type='toc']"
		  name="generate-toc">

        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' toc')"/>
            </xsl:attribute>            
            <xsl:element name="ul" namespace="{$xhtmlns}">
                <xsl:apply-templates mode='toc'
				     select="ancestor::p5:text[1]/p5:body/p5:div"/>
            </xsl:element>
	    <xsl:element name="hr" namespace="{$xhtmlns}">
	      <xsl:attribute name="style">
		<xsl:text>width: 30%;</xsl:text>
	      </xsl:attribute>
	    </xsl:element>
            <xsl:element name="ul" namespace="{$xhtmlns}">
                <xsl:apply-templates mode='toc'
				     select="ancestor::p5:text[1]/p5:back/p5:div"/>
            </xsl:element>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:body/p5:div" mode="toc">
        <xsl:element name="li" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'tocentry'"/>
            </xsl:attribute>
            <xsl:apply-templates select="p5:head" mode="toc"/>
            <xsl:if test="p5:div">
                <xsl:element name="ul" namespace="{$xhtmlns}">
                    <xsl:apply-templates mode='toc' select="p5:div"/>
                </xsl:element>
            </xsl:if>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:body/p5:div/p5:div" mode="toc">
        <xsl:element name="li" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'tocentry'"/>
            </xsl:attribute>
            <xsl:apply-templates select="p5:head" mode="toc"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:back/p5:div" mode="toc">
        <xsl:element name="li" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'tocentry'"/>
            </xsl:attribute>
            <xsl:apply-templates select="p5:head" mode="toc"/>
            <xsl:if test="p5:div">
                <xsl:element name="ul" namespace="{$xhtmlns}">
                    <xsl:apply-templates mode='toc' select="p5:div"/>
                </xsl:element>
            </xsl:if>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:back/p5:div/p5:div" mode="toc">
        <xsl:element name="li" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'tocentry'"/>
            </xsl:attribute>
            <xsl:apply-templates select="p5:head" mode="toc"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:head" mode="toc">
        <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="'toc-to-text'"/>
            </xsl:attribute>
            <xsl:attribute name="href">
                <xsl:choose>
                    <xsl:when test="../@xml:id">
                        <xsl:value-of select="concat('#', ../@xml:id)"/>
                    </xsl:when>
                    <xsl:when test="../@id">
                        <xsl:value-of select="concat('#', ../@id)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="concat('#', generate-id(..))"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
	    <xsl:if test="$num-sec = 'yes'">
	      <xsl:choose>
		<xsl:when test="ancestor::p5:front">
		</xsl:when>
		<xsl:when test="ancestor::p5:body">
		  <xsl:number count="p5:div[p5:head]"
			      level="multiple"
			      from="p5:body"
			      format="1.2.3.4"/>
		  <text>&#xA0;</text>
		</xsl:when>
		<xsl:when test="ancestor::p5:back">
		  <xsl:number count="p5:div[p5:head]"
			      level="multiple"
			      from="p5:back"
			      format="A.2.3.4"/>
		  <text>&#xA0;</text>
		</xsl:when>
	      </xsl:choose>
	    </xsl:if>
            <xsl:apply-templates select="node()"/>
        </xsl:element>
    </xsl:template>
    
    <!--* 6 back *-->
        
    
    <!--* 7 paragraphs etc. *-->
    
    <xsl:template match="p5:p" mode="fey">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' ', @type)"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:p" name="p5:p">
      <xsl:param name="footnote-backref"/>
      <xsl:variable name="first-p">
	<xsl:choose>
	  <xsl:when test="not(preceding-sibling::*)">
	    <xsl:value-of select="'first-p'"/>
	  </xsl:when>
	  <xsl:when test="preceding-sibling::*[1]/self::p5:p
			  [*
			  [self::p5:eg or self::p5:list]
			  [not(following-sibling::*)]
			  [not(following-sibling::text()[normalize-space()])
			  ]]">
	    <xsl:value-of select="'first-p'"/>
	  </xsl:when>
	  <xsl:when test="preceding-sibling::*[1]/self::p5:p">
	    <xsl:value-of select="'nonfirst-p'"/>
	  </xsl:when>
	  <xsl:when test="preceding-sibling::*[1]
			  [self::p5:head
			  or self::p5:list]">
	    <xsl:value-of select="'first-p'"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:value-of select="'nonfirst-p'"/>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:variable>
      
      <xsl:element name="div" namespace="{$xhtmlns}">
	<xsl:apply-templates select="@xml:id"/>
        <xsl:attribute name="class"><xsl:value-of
	select="concat(local-name(), ' ', $first-p)"/></xsl:attribute>
	
	<xsl:if test="$footnote-backref and
		      count(preceding-sibling::*) = 0">
	  <xsl:copy-of select="$footnote-backref"/>
	  <xsl:text> </xsl:text>
	</xsl:if>
        <xsl:apply-templates/>
    </xsl:element>
        
    </xsl:template>

    <!--* q etc. *-->
    <xsl:template match="p5:q[not(@type='block')]">
      <xsl:choose>
	<xsl:when test="count(ancestor::p5:q[not(@type='block')]) mod
			2 = 0">
	  <xsl:text>&#x201C;</xsl:text>
	  <xsl:apply-templates/>
	  <xsl:text>&#x201D;</xsl:text>
	</xsl:when>
	<xsl:when test="count(ancestor::p5:q[not(@type='block')]) mod
			2 = 1">
	  <xsl:text>&#x2018;</xsl:text>
	  <xsl:apply-templates/>
	  <xsl:text>&#x2019;</xsl:text>
	</xsl:when>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="p5:q[(@type='block')]">
      <xsl:element name="blockquote" namespace="{$xhtmlns}">
	<xsl:apply-templates/>
      </xsl:element>
    </xsl:template>

    <xsl:template match="p5:gloss">
      <xsl:choose>
	<xsl:when test="count(ancestor::p5:q[not(@type='block')]) mod
			2 = 1">
	  <xsl:text>&#x201C;</xsl:text>
	  <xsl:apply-templates/>
	  <xsl:text>&#x201D;</xsl:text>
	</xsl:when>
	<xsl:when test="count(ancestor::p5:q[not(@type='block')]) mod
			2 = 0">
	  <xsl:text>&#x2018;</xsl:text>
	  <xsl:apply-templates/>
	  <xsl:text>&#x2019;</xsl:text>
	</xsl:when>
      </xsl:choose>
    </xsl:template>

    

    <!--* notes *-->

    <xsl:template match="p5:note[@place='block']">
      <xsl:element name="div" namespace="{$xhtmlns}">
        <xsl:attribute name="class">block note</xsl:attribute>
        <xsl:apply-templates/>
      </xsl:element>        
    </xsl:template>

    <xsl:template match="p5:note[@place='foot']">
      <xsl:variable name="notenum">
	<xsl:number count="p5:note[@place='foot']" level="any"/>
      </xsl:variable>
      <xsl:element name="span" namespace="{$xhtmlns}">
        <xsl:attribute name="class">footnote-pointer</xsl:attribute>
	
	<xsl:element name="a" namespace="{$xhtmlns}">
          <xsl:attribute name="id">
	    <xsl:value-of select="concat('footnote-ref-',
				  $notenum)"/>
	  </xsl:attribute>
          <xsl:attribute name="href">
	    <xsl:value-of select="concat('#footnote-',
				  $notenum)"/>
	  </xsl:attribute>
	  <xsl:value-of select="($notenum)"/>
	</xsl:element>
      </xsl:element>
    
    </xsl:template>

    <xsl:template match="p5:note[@place='foot']" mode="notes">
      <xsl:variable name="notenum">
	<xsl:number count="p5:note[@place='foot']" level="any"/>
      </xsl:variable>
      <xsl:variable name="backref">	
	<xsl:element name="span" namespace="{$xhtmlns}">
          <xsl:attribute name="class">footnote-number</xsl:attribute>
	  <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="href">
	    <xsl:value-of select="concat('#footnote-ref-',
				  $notenum)"/>
	    </xsl:attribute>
	    <xsl:value-of select="($notenum)"/>
	  </xsl:element>
	</xsl:element>
      </xsl:variable>

      <xsl:element name="div" namespace="{$xhtmlns}">
        <xsl:attribute name="class">footnote</xsl:attribute>
        <xsl:attribute name="id">
	  <xsl:value-of select="concat('footnote-',
				$notenum)"/>
	</xsl:attribute>
	<xsl:apply-templates>
	  <xsl:with-param name="footnote-backref"
			  select="$backref"/>
	</xsl:apply-templates>
      </xsl:element>
      
    </xsl:template>
   
    <!--* lists *-->
    <xsl:template match="p5:list[not(@type) or @type = 'bullets']"
		  name="bulleted-list">
        <xsl:element name="ul" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' ', @type)"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:list[@type = 'ordered']">
        <xsl:element name="ol" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' ', @type)"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:item">
        <xsl:element name="li" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' ', @type)"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>


    <xsl:template match="p5:listBibl">
        <xsl:element name="div" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>	  
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>

    <xsl:template match="p5:listBibl/p5:bibl">
      <xsl:element name="div" namespace="{$xhtmlns}">
	<xsl:call-template name="supply-id"/>
        <xsl:attribute name="class">
	  <xsl:value-of select="local-name()"/>
	</xsl:attribute>
	<xsl:element name="span" namespace="{$xhtmlns}">
          <xsl:attribute name="class">bib-shortref</xsl:attribute>
	  <xsl:value-of select="@n"/>
	</xsl:element>
	<xsl:text> </xsl:text>
	<xsl:apply-templates/>
      </xsl:element>        
    </xsl:template>

    <xsl:template match="p5:title[@level = 'a']">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class">article <xsl:value-of
	    select="local-name()"/></xsl:attribute>
	    <xsl:text>&#x201C;</xsl:text>
            <xsl:apply-templates/>
	    <xsl:text>&#x201D;</xsl:text>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:title[@level = 's']">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class">series <xsl:value-of
	    select="local-name()"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>

    <xsl:template match="p5:date[ancestor::p5:bibl]
			 | p5:biblScope">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>	  
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>

    <!--* tables *-->
    <xsl:template match="p5:table">
        <xsl:element name="table" namespace="{$xhtmlns}">	  
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:row">
        <xsl:element name="tr" namespace="{$xhtmlns}">	  
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:cell">
        <xsl:element name="td" namespace="{$xhtmlns}">	  
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:row[@role='label']/p5:cell
			 | p5:cell[@role='label']">
        <xsl:element name="th" namespace="{$xhtmlns}">	  
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <!--* examples *-->
    <xsl:template match="p5:eg">
        <xsl:element name="pre" namespace="{$xhtmlns}">	  
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>

    <!--* figures *-->
    

  <xsl:template match="p5:figure">
    <xsl:element name="div" namespace="{$xhtmlns}">
      <xsl:attribute name="class">figure</xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="p5:figure/p5:graphic">
    <xsl:element name="img" namespace="{$xhtmlns}">
      <xsl:attribute name="src"><xsl:value-of select="@url"/></xsl:attribute>
      <xsl:choose>	
	<xsl:when test="@rend and
			normalize-space(translate(@rend,'0123456789.%',
			' '))
			= ''">
	  <xsl:attribute name="style">
	    <xsl:value-of select="concat('width: ', @rend,
				  '; text-align: center;')"/>
	  </xsl:attribute>
	</xsl:when>
	<xsl:when test="ancestor::p5:p[@rend = 'slide']">
	  <xsl:attribute name="style">
	    <xsl:text>width: 40%; float: right; padding-right: 1em;</xsl:text>
	  </xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:attribute name="style">
	    <xsl:text>width: 80%;</xsl:text>
	  </xsl:attribute>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
    
    <!--* 8 phrase-level markup *-->
    
    <xsl:template match="p5:label">
        <xsl:element name="b" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="concat(name(), ' ', @type)"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:seg">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:idno">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    
    <xsl:template match="p5:lb">
        <xsl:element name="br" namespace="{$xhtmlns}"/>          
    </xsl:template>
    
    <xsl:template match="p5:ref">
        <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="href">
                <xsl:value-of select="@target"/>
            </xsl:attribute>
            <xsl:choose>
                <xsl:when test="normalize-space(string(.)) = '' and not(*)">
                    <xsl:choose>
                        <xsl:when test="@type='home'">Home</xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="concat('[', @href, ']')"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
    </xsl:template>

    <xsl:template match="p5:ptr[starts-with(@target,'#')]">
      <xsl:variable name="refid" select="substring-after(@target,'#')"/>
      <xsl:variable name="target" select="//*[@xml:id=$refid]"/>
      <xsl:choose>
	<xsl:when test="@type='bibl' or $target/self::p5:bibl">
          <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:attribute name="href"><xsl:value-of select="@target"/></xsl:attribute>
            <xsl:value-of select="$target/@n"/>
        </xsl:element>        
	</xsl:when>
	<xsl:when test="@type='sectitle' or $target/self::p5:div/p5:head">
          <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:attribute name="href"><xsl:value-of select="@target"/></xsl:attribute>
            <xsl:value-of select="$target/p5:head[1]"/>
        </xsl:element>        
	</xsl:when>
	<xsl:otherwise>
          <xsl:element name="a" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:attribute name="href"><xsl:value-of select="@target"/></xsl:attribute>
            <xsl:text>[ptr]</xsl:text>
        </xsl:element>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    
    <!--* italics *-->
    <xsl:template match="p5:hi 
			 | p5:foreign
			 | p5:ident
			 | p5:title[not(@level)]  
			 | p5:title[(@level = 'm')]  
			 | p5:title[(@level = 'j')]  
			 | p5:mentioned
			 | p5:term
			 | p5:emph">
        <xsl:element name="i" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:soCalled">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:text>&#x2018;</xsl:text>
            <xsl:apply-templates/>
            <xsl:text>&#x2019;</xsl:text>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:gi|p5:att|p5:code|p5:kw">
        <xsl:element name="tt" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:tag">
        <xsl:element name="tt" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of
	    select="local-name()"/></xsl:attribute>
	    <xsl:text>&lt;</xsl:text>
            <xsl:apply-templates/>
	    <xsl:text>&gt;</xsl:text>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:author">
        <xsl:element name="span" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>

    <!--* subscripts, superscripts, other special highlighting *-->
    
    <xsl:template match="p5:hi[@rend='sub']">
        <xsl:element name="sub" namespace="{$xhtmlns}">
            <xsl:attribute name="class"><xsl:value-of select="local-name()"/></xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:hi[@rend='bold']">
      <xsl:message>bobobo!</xsl:message>
        <xsl:element name="b" namespace="{$xhtmlns}">
            <xsl:apply-templates/>
        </xsl:element>        
    </xsl:template>
    
    <xsl:template match="p5:add">
        <xsl:element name="ins" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="p5:del">
        <xsl:element name="del" namespace="{$xhtmlns}">
            <xsl:attribute name="class">
                <xsl:value-of select="name()"/>
            </xsl:attribute>
            <xsl:apply-templates/>
        </xsl:element>
    </xsl:template>
    
    <!--* toc mode *-->

    <xsl:template name="supply-id">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@xml:id">
            <xsl:value-of select="@xml:id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </xsl:template>

    <xsl:template match="@xml:id">
      <xsl:attribute name="id">
	<xsl:value-of select="."/>
      </xsl:attribute>
    </xsl:template>
    
</xsl:stylesheet>
